@@ -18,7 +18,7 @@
 #
 #  SYNOPSIS
 #
-#  survkit(times, censor=NULL, ccov=NULL, tccov=NULL,
+#  survkit(times, censor=NULL, ccov=NULL, tvcov=NULL,
 #	strata=NULL, id=NULL, model="Weibull", baseline=F,
 #	residuals=F, survival=NULL, svalues=NULL, valrho=NULL,
 #	constraints=NULL, impose=NULL, dist=NULL, random=NULL,
@@ -30,10 +30,13 @@
 #
 #    Function to fit  Weibull and Cox models with frailties
 
-library.dynam("event")
+.First.lib <- function(lib, pkg) {
+	library.dynam("event", pkg, lib)
+	provide(event)
+}
 require(rmutil)
 
-survkit <- function(times, censor=NULL, ccov=NULL, tccov=NULL,
+survkit <- function(times, censor=NULL, ccov=NULL, tvcov=NULL,
 	strata=NULL, id=NULL, model="Weibull", baseline=F,
 	residuals=F, survival=NULL, svalues=NULL, valrho=NULL,
 	constraints=NULL, impose=NULL, dist=NULL, random=NULL,
@@ -82,7 +85,6 @@ survkit <- function(times, censor=NULL, ccov=NULL, tccov=NULL,
 		nunique <- length(unique(id))}
 	else nunique <- nrr
 	info <- cbind(times,censor)
-
 	squant <- xtabl <- itabl <- fact <- cont <- xlabelc <- xtabl2 <-
 		itabl2 <- info2 <- xlabelf <- xlabelrg <- ind1 <- ind2 <-
 		xlabelrn <- istime <- NULL
@@ -93,7 +95,7 @@ survkit <- function(times, censor=NULL, ccov=NULL, tccov=NULL,
 # covariates
 # model : assume all variables in model
 	if(!is.null(ccov)){
-		if(!is.language(ccov))stop("ccov must be a model formula")
+		if(!inherits(ccov,"formula"))stop("ccov must be a model formula")
 		mf <- model.frame(terms(ccov),sys.frame(sys.parent()),
 			na.action=na.fail)
 		if(nrow(mf)!=nunique&&nrow(mf)!=nrr)stop("time-constant covariates must have one value per individual or one per time")
@@ -116,9 +118,9 @@ survkit <- function(times, censor=NULL, ccov=NULL, tccov=NULL,
 	if(model!="Weibull"){
 		iplus1 <- rep(0,nccov)
 		iplus2 <- rep(0,ndcov)}
-	if(!is.null(tccov)){
-		if(!is.language(tccov))stop("tccov must be a model formula")
-		mf <- model.frame(terms(tccov),sys.frame(sys.parent()),
+	if(!is.null(tvcov)){
+		if(!inherits(tvcov,"formula"))stop("tvcov must be a model formula")
+		mf <- model.frame(terms(tvcov),sys.frame(sys.parent()),
 			na.action=na.fail)
 		if(nrow(mf)!=nrr)stop("time-constant covariates must have one value per time")
 		for(i in 1:ncol(mf)){
@@ -291,7 +293,7 @@ survkit <- function(times, censor=NULL, ccov=NULL, tccov=NULL,
 	if(model=="Cox")icrho <- 0
 	else if(!is.null(valrho)){
 		icrho <- 0
-		if(is.vector(valrho,mode="double")&&length(valrho)!=nstrata)
+		if(is.vector(valrho,mode="numeric")&&length(valrho)!=nstrata)
 			stop("valrho must have a value for each	stratum")
 		valrho <- c(valrho,rep(0,mxstra-nstrata))}
 	else {
@@ -306,7 +308,7 @@ survkit <- function(times, censor=NULL, ccov=NULL, tccov=NULL,
 		survival <- match.arg(survival,c("equal","quantiles","specific","all.times"))
 		if(survival!="all.times"){
 			if(is.null(svalues))stop("svalues must be supplied with survival")
-			if(!is.vector(svalues,mode="double"))stop("svalues must be a vector")
+			if(!is.vector(svalues,mode="numeric"))stop("svalues must be a vector")
 			if(length(svalues)>nstimax)stop("too many svalues")
 			if(survival!="equal"&&any(diff(svalues)<=0))
 				stop("svalues must be strictly increasing")}
@@ -388,7 +390,8 @@ survkit <- function(times, censor=NULL, ccov=NULL, tccov=NULL,
 		xmom=double(4),
 		surv=double(3*nstimax*nrr),
 		like=double(2),
-		df=integer(2))
+		df=integer(2),
+		DUP=F)
 	else 
 		.Fortran("cox",
 		iconst=as.integer(iconst),
@@ -452,11 +455,12 @@ survkit <- function(times, censor=NULL, ccov=NULL, tccov=NULL,
 			colnames(base) <- c("cumulative hazard","survival function")}
 		else base <- NULL
 		if(residuals){
-			resid <- matrix(z0$resid,ncol=3)
-			resid <- resid[resid[,1]>0,1:3]
 			o <- order(info2[,1])
 			info2 <- info2[o,]
-			resid <- cbind(info2[info2[,2]>-1,1:3],resid)
+			resid <- matrix(z0$resid,ncol=3)
+			o <- resid[,1]>0
+			resid <- resid[resid[,1]>0,1:3]
+			resid <- cbind(info2[info2[,2]>-1&o,1:3],resid)
 			colnames(resid) <- c("time","censor","stratum","generalized","martingale","deviance")}
 		else resid <- NULL}
 	else resid <- base <- km <- NULL
@@ -543,7 +547,7 @@ survkit <- function(times, censor=NULL, ccov=NULL, tccov=NULL,
 		ndim <- ndim-ndimg-ndimn}
 	z <- list(
 		ccov=ccov,
-		tccov=tccov,
+		tvcov=tvcov,
 		beta=beta,
 		betag=betag,
 		betan=betan,
@@ -591,11 +595,13 @@ residuals.survivalkit <- function(z){
 		if(!is.null(z$residuals))print(z$residuals)
 		else print("residuals not available: use residuals=T")}
 	else print("residuals only available for Cox model")}
+baseline <- function(z, ...) UseMethod("baseline")
 baseline.survivalkit <- function(z){
 	if(inherits(z,"Cox")){
 		if(!is.null(z$baseline))print(z$baseline)
 		else print("baseline not available: use basellllline=T")}
 	else print("baseline only available for Cox model")}
+survival <- function(z, ...) UseMethod("survival")
 survival.survivalkit <- function(z){
 	if(!is.null(z$survival))print(z$survival)
 	else print("survival not available: use survival option")}
